@@ -1,6 +1,6 @@
 /*
  * backend.h -- Interface exported by XBoard back end
- * $Id: backend.h,v 1.43 2000/09/18 00:15:02 mann Exp $
+ * $Id: backend.h,v 1.46 2001/02/05 20:32:32 mann Exp $
  *
  * Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts.
  * Enhancements Copyright 1992-95 Free Software Foundation, Inc.
@@ -56,7 +56,7 @@ extern GameMode gameMode;
 extern int pausing, cmailMsgLoaded, flipView;
 extern char white_holding[], black_holding[];
 extern int currentMove, backwardMostMove, forwardMostMove;
-extern int blackPlaysFirst, analysisSupport;
+extern int blackPlaysFirst;
 extern FILE *debugFP;
 extern char* programVersion;
 extern ProcRef firstProgramPR, secondProgramPR;
@@ -98,6 +98,7 @@ void ForwardEvent P((void));
 void BackwardEvent P((void));
 void ToEndEvent P((void));
 void ToStartEvent P((void));
+void ToNrEvent P((int to));
 void RevertEvent P((void));
 void RetractMoveEvent P((void));
 void MoveNowEvent P((void));
@@ -197,4 +198,44 @@ char *GameListLine P((int, GameInfo *));
 extern char* StripHighlight P((char *));  /* returns static data */
 extern char* StripHighlightAndTitle P((char *));  /* returns static data */
 
+
+typedef struct _CPS {
+    char *which;
+    int maybeThinking;
+    ProcRef pr;
+    InputSourceRef isr;
+    char *twoMachinesColor; /* "white\n" or "black\n" */
+    char *program;
+    char *host;
+    char *dir;
+    struct _CPS *other;
+    char *initString;
+    char *computerString;
+    int sendTime; /* 0=don't, 1=do, 2=test */
+    int sendDrawOffers;
+    int useSigint;
+    int useSigterm;
+    int offeredDraw; /* countdown */
+    int reuse;
+    int useSetboard; /* 0=use "edit"; 1=use "setboard" */
+    int useSAN;      /* 0=use coordinate notation; 1=use SAN */
+    int usePing;     /* 0=not OK to use ping; 1=OK */
+    int lastPing;
+    int lastPong;
+    int usePlayother;/* 0=not OK to use playother; 1=OK */
+    int useColors;   /* 0=avoid obsolete white/black commands; 1=use them */
+    int useUsermove; /* 0=just send move; 1=send "usermove move" */
+    int sendICS;     /* 0=don't use "ics" command; 1=do */
+    int sendName;    /* 0=don't use "name" command; 1=do */
+    int sdKludge;    /* 0=use "sd DEPTH" command; 1=use "depth\nDEPTH" */
+    int stKludge;    /* 0=use "st TIME" command; 1=use "level 1 TIME" */
+    char tidy[MSG_SIZ];
+    int matchWins;
+    char variants[MSG_SIZ];
+    int analysisSupport;
+    int protocolVersion;
+} ChessProgramState;
+
+extern ChessProgramState first, second;
+
 #endif /* _BACKEND */
