@@ -1,6 +1,6 @@
 /*
  * lists.c -- Functions to implement a double linked list
- * XBoard $Id: lists.c,v 1.3 1998/05/24 05:14:31 mann Exp $
+ * XBoard $Id: lists.c,v 1.5 2000/12/17 01:55:48 mann Exp $
  *
  * Copyright 1995 Free Software Foundation, Inc.
  *
@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
  * ------------------------------------------------------------------------
  *
  * This file could well be a part of backend.c, but I prefer it this
@@ -90,7 +90,7 @@ ListNode *ListNodeCreate(s)
 {
     ListNode *n;
 
-    if ((n = malloc(s))) {
+    if ((n = (ListNode*) malloc(s))) {
 	n->succ = NULL; /*  Mark node as not being member of a list.    */
 	n->pred = NULL;
     }
