@@ -1,6 +1,6 @@
 /*
  * zippy.c -- Implements Zippy the Pinhead chess player on ICS in XBoard
- * $Id: zippy.c,v 1.58 2000/09/17 23:49:27 mann Exp $
+ * $Id: zippy.c,v 1.60 2001/02/04 04:42:43 mann Exp $
  *
  * Copyright 1991 by Digital Equipment Corporation, Maynard, Massachusetts.
  * Enhancements Copyright 1992-95 Free Software Foundation, Inc.
@@ -65,7 +65,6 @@ extern char *getenv();
 #  include <strings.h>
 # endif /* not HAVE_STRING_H */
 #endif /* not STDC_HEADERS */
-#define HI "hlelo "
 
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -77,16 +76,7 @@ extern char *getenv();
 #  include <time.h>
 # endif
 #endif
-
-#if !HAVE_RANDOM
-# if HAVE_RAND48
-#  define srandom srand48
-#  define random lrand48
-# else /* not HAVE_RAND48 */
-#  define srandom srand
-#  define random rand
-# endif /* not HAVE_RAND48 */
-#endif /* !HAVE_RANDOM */
+#define HI "hlelo "
 
 #if HAVE_UNISTD_H
 # include <unistd.h>
@@ -197,6 +187,7 @@ void ZippyInit()
     if (p != NULL) {
       appData.zippyVariants = p;
     }
+    strcpy(first.variants, appData.zippyVariants);
 
     srandom(time(NULL));
 }
@@ -342,7 +333,8 @@ void Speak(how, whom)
     }
     done = FALSE;
 
-    strcpy(zipbuf, how);
+    strcpy(zipbuf, ics_prefix);
+    strcat(zipbuf, how);
     strcat(zipbuf, " ");
     if (whom != NULL) {
 	strcat(zipbuf, whom);
@@ -402,10 +394,10 @@ int ZippyControl(buf, i)
 	     StrCaseCmp(player, gameInfo.black) == 0) ||
 	    (gameMode == IcsPlayingBlack &&
 	     StrCaseCmp(player, gameInfo.white) == 0)) {
-	    SendToICS("say This computer does not play Crafty clones\n");
-	    SendToICS("abort\n");
-	    sprintf(reply, "+noplay %s\n", player);
-	    SendToICS(reply);
+
+	  sprintf(reply, "%ssay This computer does not play Crafty clones\n%sabort\n%s+noplay %s\n",
+		  ics_prefix, ics_prefix, ics_prefix, player);
+	  SendToICS(reply);
 	}
 	return TRUE;
     }
@@ -433,18 +425,18 @@ int ZippyControl(buf, i)
 	 looking_at(buf, i, "* tells you: [automatic message] I chose you"))) {
 	player = StripHighlightAndTitle(star_match[0]);
 	if (appData.zippyBughouse > 1) {
-	    sprintf(reply, "partner %s\n", player);
+	    sprintf(reply, "%spartner %s\n", ics_prefix, player);
 	    SendToICS(reply);
 	    if (strcmp(zippyPartner, player) != 0) {
 		strcpy(zippyPartner, player);
 		SendToProgram(reply, &first);
 	    }
 	} else if (appData.zippyBughouse > 0) {
-	    sprintf(reply, "decline %s\n", player);
+	    sprintf(reply, "%sdecline %s\n", ics_prefix, player);
 	    SendToICS(reply);
 	} else {
-	    sprintf(reply, "tE %s This computer cannot play bughouse\n",
-		    player);
+	    sprintf(reply, "%stell %s This computer cannot play bughouse\n",
+		    ics_prefix, player);
 	    SendToICS(reply);
 	}
 	return TRUE;
@@ -527,11 +519,13 @@ int ZippyControl(buf, i)
 	    SendToProgram("\n", &first);
 	} else if (strncmp(star_match[1], HI, 6) == 0) {
 	    extern char* programVersion;
-	    sprintf(reply, "teL %s %s\n", player, programVersion);
+	    sprintf(reply, "%stell %s %s\n",
+		    ics_prefix, player, programVersion);
 	    SendToICS(reply);
 	} else if (strncmp(star_match[1], "W0W!! ", 6) == 0) {
 	    extern char* programVersion;
-	    sprintf(reply, "Tel %s %s\n", player, programVersion);
+	    sprintf(reply, "%stell %s %s\n", ics_prefix,
+		    player, programVersion);
 	    SendToICS(reply);
 	} else if (appData.zippyTalk && ((random() % 10) < 9)) {
 	    if (strcmp(player, ics_handle) != 0) {
@@ -554,54 +548,51 @@ int ZippyConverse(buf, i)
      int *i;
 {
     static char lastgreet[MSG_SIZ];
+    char reply[MSG_SIZ];
     int oldi;
 
     /* Shouts and emotes */
     if (looking_at(buf, i, "--> * *") ||
 	looking_at(buf, i, "* shouts: *")) {
+      if (appData.zippyTalk) {
 	char *player = StripHighlightAndTitle(star_match[0]);
 	if (strcmp(player, ics_handle) == 0) {
 	    return TRUE;
 	} else if (appData.zippyPinhead[0] != NULLCHAR &&
 		   StrCaseStr(star_match[1], appData.zippyPinhead) != NULL) {
-	    SendToICS("insult ");
-	    SendToICS(player);
-	    SendToICS("\n");
+	    sprintf(reply, "insult %s\n", player);
+	    SendToICS(reply);
 	} else if (ZippyCalled(star_match[1])) {
 	    Speak("shout", NULL);
 	}
-	return TRUE;
+      }
+      return TRUE;
     }
 
-    if (looking_at(buf, i, "* kibitzes: *") && ((random() % 10) < 9)) {
+    if (looking_at(buf, i, "* kibitzes: *")) {
+      if (appData.zippyTalk && (random() % 10) < 9) {
 	char *player = StripHighlightAndTitle(star_match[0]);
-
 	if (strcmp(player, ics_handle) != 0) {
 	    Speak("kibitz", NULL);
-	    return TRUE;
 	}
+      }
+      return TRUE;
     }
 
-    if (looking_at(buf, i, "* whispers: *") && ((random() % 10) < 9)) {
+    if (looking_at(buf, i, "* whispers: *")) {
+      if (appData.zippyTalk && (random() % 10) < 9) {
 	char *player = StripHighlightAndTitle(star_match[0]);
-
 	if (strcmp(player, ics_handle) != 0) {
 	    Speak("whisper", NULL);
-	    return TRUE;
 	}
-    }
-
-    if ((looking_at(buf, i, "You have * message") &&
-	 atoi(star_match[0]) != 0) ||
-	looking_at(buf, i, "* has left a message for you") ||
-	looking_at(buf, i, "* just sent you a message")) {
-	SendToICS("messages\nclearmessages\nclearmessages *\n");
-	return TRUE;
+      }
+      return TRUE;
     }
 
     /* Messages */
     if ((looking_at(buf, i, ". * (*:*): *") && isdigit(star_match[1][0])) ||
 	 looking_at(buf, i, ". * at *:*: *")) {
+      if (appData.zippyTalk) {
 	FILE *f;
 	char *player = StripHighlightAndTitle(star_match[0]);
 
@@ -613,7 +604,8 @@ int ZippyConverse(buf, i)
 		    star_match[1], star_match[2], star_match[3]);
 	    fclose(f);
 	}
-	return TRUE;
+      }
+      return TRUE;
     }
 
     /* Channel tells */
@@ -628,25 +620,39 @@ int ZippyConverse(buf, i)
 	    *i = oldi;
 	    return FALSE;
 	}
-	player = StripHighlightAndTitle(star_match[0]);
-	channel = strrchr(star_match[1], '(');
-	if (channel == NULL) {
-	  channel = star_match[1];
-	} else {
-	  channel++;
-	}
-	channel[strlen(channel)-1] = NULLCHAR;
+	if (appData.zippyTalk) {
+	  player = StripHighlightAndTitle(star_match[0]);
+	  channel = strrchr(star_match[1], '(');
+	  if (channel == NULL) {
+	    channel = star_match[1];
+	  } else {
+	    channel++;
+	  }
+	  channel[strlen(channel)-1] = NULLCHAR;
 #if 0
-	/* Always tell to the channel (probability 90%) */
-	if (strcmp(player, ics_handle) != 0 && (random() % 10) < 9) {
+	  /* Always tell to the channel (probability 90%) */
+	  if (strcmp(player, ics_handle) != 0 && (random() % 10) < 9) {
 	    Speak("tell", channel);
-	}
+	  }
 #else
-	/* Tell to the channel only if someone mentions our name */
-	if (ZippyCalled(star_match[2])) {
+	  /* Tell to the channel only if someone mentions our name */
+	  if (ZippyCalled(star_match[2])) {
 	    Speak("tell", channel);
-	}
+	  }
 #endif
+	}
+	return TRUE;
+    }
+
+    if (!appData.zippyTalk) return FALSE;
+
+    if ((looking_at(buf, i, "You have * message") &&
+	 atoi(star_match[0]) != 0) ||
+	looking_at(buf, i, "* has left a message for you") ||
+	looking_at(buf, i, "* just sent you a message")) {
+        sprintf(reply, "%smessages\n%sclearmessages *\n",
+		ics_prefix, ics_prefix);
+	SendToICS(reply);
 	return TRUE;
     }
 
@@ -654,9 +660,8 @@ int ZippyConverse(buf, i)
 	if ((random() % 3) == 0) {
 	    char *player = StripHighlightAndTitle(star_match[0]);
 	    strcpy(lastgreet, player);
-	    SendToICS("greet ");
-	    SendToICS(player);
-	    SendToICS("\n");
+	    sprintf(reply, "greet %s\n", player);
+	    SendToICS(reply);
 	    Speak("tell", player);
 	}
     }	
@@ -664,18 +669,16 @@ int ZippyConverse(buf, i)
     if (looking_at(buf, i, "Notification: * has departed")) {
 	if ((random() % 3) == 0) {
 	    char *player = StripHighlightAndTitle(star_match[0]);
-	    SendToICS("farewell ");
-	    SendToICS(player);
-	    SendToICS("\n");
+	    sprintf(reply, "farewell %s\n", player);
+	    SendToICS(reply);
 	}
     }	
 
     if (looking_at(buf, i, "Not sent -- * is censoring you")) {
 	char *player = StripHighlightAndTitle(star_match[0]);
 	if (strcmp(player, lastgreet) == 0) {
-	    SendToICS("-notify ");
-	    SendToICS(player);
-	    SendToICS("\n");
+	    sprintf(reply, "%s-notify %s\n", ics_prefix, player);
+	    SendToICS(reply);
 	}
     }	
 
@@ -726,15 +729,16 @@ void ZippyHandleChallenge(srated, swild, sbase, sincrement, opponent)
     base = atoi(sbase);
     increment = atoi(sincrement);
 
-    /* If desired, you can insert more code here to decline matches based
-       on rated, variant, base, and increment.  Otherwise they are mostly
-       not used.  It is probably best to do this with formulas anyway... */
+    /* If desired, you can insert more code here to decline matches
+       based on rated, variant, base, and increment, but it is
+       easier to use the ICS formula feature instead. */
 
     if ((variant == VariantLoadable) || 
 	(StrStr(appData.zippyVariants, varname) == NULL)) {
         sprintf(buf,
-	    "telL %s This computer cannot play %s [%s], only %s\ndecline %s\n",
-		opponent, swild, varname, appData.zippyVariants, opponent);
+	 "%stell %s This computer can't play %s [%s], only %s\n%sdecline %s\n",
+		ics_prefix, opponent, swild, varname, appData.zippyVariants,
+		ics_prefix, opponent);
 	SendToICS(buf);
 	return;
     }
@@ -752,12 +756,22 @@ void ZippyHandleChallenge(srated, swild, sbase, sincrement, opponent)
 	strcmp(opponent, zippyLastOpp) == 0 &&
 	zippyConsecGames >= appData.zippyMaxGames &&
 	difftime(time(0), zippyLastGameEnd) < appData.zippyReplayTimeout) {
-      sprintf(buf, "tell %s Sorry, you have just played %d consecutive games against %s.  To give others a chance, please wait %d seconds or until someone else has played.\ndecline %s\n", opponent, zippyConsecGames, ics_handle, appData.zippyReplayTimeout, opponent);
+      sprintf(buf, "%stell %s Sorry, you have just played %d consecutive games against %s.  To give others a chance, please wait %d seconds or until someone else has played.\n%sdecline %s\n",
+	      ics_prefix, opponent, zippyConsecGames, ics_handle,
+	      appData.zippyReplayTimeout, ics_prefix, opponent);
       SendToICS(buf);
       return;
     }
 
-    sprintf(buf, "accept %s\n", opponent);
+    /* Engine still thinking about last game? */
+    if (first.lastPing != first.lastPong) {
+      sprintf(buf, "%stell %s Oops, I'm not quite ready for a new game yet.\n",
+	      ics_prefix, opponent);
+      SendToICS(buf);
+      return;
+    }
+
+    sprintf(buf, "%saccept %s\n", ics_prefix, opponent);
     SendToICS(buf);
     if (appData.zippyTalk) {
       Speak("tell", opponent);
@@ -848,8 +862,10 @@ int ZippyMatch(buf, i)
 	if (appData.zippyAbort ||
 	    (gameMode == IcsPlayingWhite && whiteTimeRemaining < 0) ||
 	    (gameMode == IcsPlayingBlack && blackTimeRemaining < 0)) {
+	    SendToICS(ics_prefix);
 	    SendToICS("abort\n");
 	} else {
+	    SendToICS(ics_prefix);
 	    if (appData.zippyTalk)
 	      SendToICS("say Whoa no!  I am having FUN!!\n");
 	    else
@@ -861,8 +877,10 @@ int ZippyMatch(buf, i)
     if (looking_at(buf, i, "requests adjournment") ||
 	looking_at(buf, i, "would like to adjourn")) {
       if (appData.zippyAdjourn) {
+	SendToICS(ics_prefix);
 	SendToICS("adjourn\n");
       } else {
+	SendToICS(ics_prefix);
 	if (appData.zippyTalk)
 	  SendToICS("say Whoa no!  I am having FUN playing NOW!!\n");
 	else
@@ -891,7 +909,7 @@ void ZippyFirstBoard(moveNum, basetime, increment)
     }
 
     if (startedFromSetupPosition || (!appData.getMoveList && moveNum > 0)) {
-	SendBoard(&first, boards[moveNum], !WhiteOnMove(moveNum));
+	SendBoard(&first, moveNum);
     }
 
     sprintf(buf, "level 0 %d %d\n", basetime, increment);
@@ -909,20 +927,22 @@ void ZippyFirstBoard(moveNum, basetime, increment)
        we've been gathering. */
     for (w=0; w<num_opps; w++) {
 	if (!strcmp(opp_name[w], opp)) {
-	    SendToProgram("computer\n",&first);
+	    SendToProgram(first.computerString, &first);
 	    break;
 	}
     }
 
     /* Ratings might be < 0 which means "we haven't seen a ratings
-       message from ICS." Send Crafty 0 in that case */
+       message from ICS." Send 0 in that case */
     w = (gameInfo.whiteRating >= 0) ? gameInfo.whiteRating : 0;
     b = (gameInfo.blackRating >= 0) ? gameInfo.blackRating : 0;
     
     firstMove = FALSE;
     if (gameMode == IcsPlayingWhite) {
-	sprintf(buf, "name %s\n", gameInfo.black);  /* for Crafty */
-	SendToProgram(buf, &first);
+        if (first.sendName) {
+	  sprintf(buf, "name %s\n", gameInfo.black);
+	  SendToProgram(buf, &first);
+	}
 	strcpy(ics_handle, gameInfo.white);
 	sprintf(buf, "rating %d %d\n", w, b);
 	SendToProgram(buf, &first);
@@ -930,23 +950,40 @@ void ZippyFirstBoard(moveNum, basetime, increment)
 	    /* Resume from adjournments with move list, to be sent later */
 	    if (moveNum == 0) {
 		/* Not an adjournment */
-		SendToProgram("black\n", &first); /*gnu kludge*/
-		SendTimeRemaining(&first, TRUE);
-		SendToProgram("white\ngo\n", &first);
+		if (first.useColors) {
+		  SendToProgram("black\n", &first); /*gnu kludge*/
+		  SendTimeRemaining(&first, TRUE);
+		  SendToProgram("white\ngo\n", &first);
+		} else {
+		  SendTimeRemaining(&first, TRUE);
+		  SendToProgram("go\n", &first);
+		}
 	    }
 	} else {
 	    /* Resume from adjournments with position only, sent above */
 	    if (WhiteOnMove(moveNum)) {
-		SendToProgram("black\n", &first); /*gnu kludge*/
-		SendTimeRemaining(&first, TRUE);
-		SendToProgram("white\ngo\n", &first);
+		if (first.useColors) {
+		  SendToProgram("black\n", &first); /*gnu kludge*/
+		  SendTimeRemaining(&first, TRUE);
+		  SendToProgram("white\ngo\n", &first);
+		} else {
+		  SendTimeRemaining(&first, TRUE);
+		  SendToProgram("go\n", &first);
+		}
 	    } else {
-		firstMove = TRUE; /* need to send whiteString later */
+	        if (first.usePlayother) {
+		  SendToProgram("playother\n", &first);
+		  firstMove = FALSE;
+		} else {
+		  firstMove = TRUE;
+		}
 	    }
 	}
     } else if (gameMode == IcsPlayingBlack) {
-	sprintf(buf, "name %s\n", gameInfo.white);
-	SendToProgram(buf, &first);
+        if (first.sendName) {
+	  sprintf(buf, "name %s\n", gameInfo.white);
+	  SendToProgram(buf, &first);
+	}
 	strcpy(ics_handle, gameInfo.black);
 	sprintf(buf, "rating %d %d\n", b, w);
 	SendToProgram(buf, &first);
@@ -956,11 +993,20 @@ void ZippyFirstBoard(moveNum, basetime, increment)
 	} else {
 	    /* Resume from adjournments with position only, sent above */
 	    if (!WhiteOnMove(moveNum)) {
-		SendToProgram("white\n", &first); /*gnu kludge*/
-		SendTimeRemaining(&first, FALSE);
-		SendToProgram("black\ngo\n", &first);
+		if (first.useColors) {
+		  SendToProgram("white\n", &first); /*gnu kludge*/
+		  SendTimeRemaining(&first, FALSE);
+		  SendToProgram("black\ngo\n", &first);
+		} else {
+		  SendTimeRemaining(&first, FALSE);
+		  SendToProgram("go\n", &first);
+		}
 	    } else {
-		/* nothing to do here */
+	        if (first.usePlayother) {
+		  SendToProgram("playother\n", &first);
+		} else {
+		  /* nothing to do here */
+		}
 	    }
 	}
     }
